/*
 Data acquisition structures and constants
 
 Copyright (c) 2008 Dimtel, Inc., All Rights Reserved

 $Id: dacq.h,v 1.1.1.1 2009/06/02 20:25:59 dim Exp $
 $Date: 2009/06/02 20:25:59 $
 $Author: dim $
 $Revision: 1.1.1.1 $
 $Log: dacq.h,v $
 Revision 1.1.1.1  2009/06/02 20:25:59  dim
 Initial import


*/

#ifndef _DACQ_H_
#define _DACQ_H_

#include <pthread.h>

#define ADC_LENGTH       24576
#define LPF_LENGTH 	 10
#define GET_DEMUX(a) 	 (4 + 2*a)

/* Wait for acquisitions shorter than this */
#define T_ACQ_WAIT_MAX	30.0
/* Wait for this many uS*/
#define T_ACQ_WAIT	20000

typedef struct acq_data
{
  int          adc[ADC_LENGTH];
  unsigned int *data;
  char new;
} acq_data;

#define TRIG_NEW	0x1
#define TRIG_PROC	0x2
#define T_PROC_WAIT	1000

/* Data acquisition modes */
#define DACQ_MODE_RAW	0
#define DACQ_MODE_BXB	1

struct proc_raw
{
  char first, scale_update;
  double us_per_sample;
  long   *adc;
  double tmp[ADC_LENGTH];
  double *mag, mag_old[ADC_LENGTH/2+1];
  float *freq;
};  

struct proc_bxb
{
  char first, scale_update, stop;
  int    samples_per_turn;
  long   adc[ADC_LENGTH];
  double us_per_sample;
  char   *mask;
  double *mag, *mean, *rms, *mag_old, *tmp;
  double *data;
  float  *tsc, *freq;
  int    *max_rms;
};

typedef struct proc_data
{
  char   mode;
  double alpha;
  
  struct proc_raw raw;
  struct proc_bxb bxb;
} proc_data;

#endif /* _DACQ_H_ */
