/*
 Convert text string into bunch enabling bit string
 
 Copyright (c) 2007 Dimtel, Inc., All Rights Reserved

 $Id: pattern.c,v 1.2 2009/02/09 07:41:27 dim Exp $
 $Date: 2009/02/09 07:41:27 $
 $Author: dim $
 $Revision: 1.2 $
 $Log: pattern.c,v $
 Revision 1.2  2009/02/09 07:41:27  dim
 Pattern inversion added, bunch cleaning support

 Revision 1.1  2007/12/30 15:27:06  dim
 Phase servo, drive pattern generation.


*/

#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>

enum tokens {
  TOKEN_INT,
  TOKEN_RANGE
};

static char *patternToken(char *pattern, int *low, int *high, 
		     int *step, int *type);


int patternConvert(char *pattern, char *bunches, int hnum)
{
  char *next = pattern, bit_set = 1, bit_clr = 0;
  int low, high, step, type, i;

  /* Check for inversion upfront */
  if (*next == '!')
  {
    next++;
    bit_set = 0;
    bit_clr = 1;
  }

  /* Clear the output string */
  for (i = 0; i < hnum; i++)
    bunches[i] = bit_clr;

  /* Nothing to do */
  if (*next=='\0')
    return 0;
      
  while (next!=NULL)
  {
    next=patternToken(next, &low, &high, &step, &type);
    if (type < 0) /* Error!!! */
      return -type;

    if (type==TOKEN_INT)
    {
      if (low <= hnum && low >= 1) /* Within the range */
	bunches[low-1] = bit_set;
    }
    else
    {
      if (low > hnum || low < 1 || high > hnum || high < 1)
        break; /* Skip */

      if (step==0) 
        step=1;

      if (low>high)
      {
	for (i=low; i <= hnum; i+=step)
	  bunches[(i-1)] = bit_set;
	i-=hnum;
	for (; i<=high; i+=step)
	  bunches[(i-1)] = bit_set;
      }
      else
        for (i=low; i<=high; i+=step)
          bunches[(i-1)] = bit_set;
    }
  }
  return 0;
}

char *patternToken(char *pattern, int *low, int *high, 
		     int *step, int *type)
{
  char *p, *next;
  int step_flg=0;
  
  p=pattern; *type=TOKEN_INT;
	  
  while(!isspace(*p) && *p!='\0')
  {
    if (*p==':')
    {
      if (*type==TOKEN_RANGE) /* second : */
        step_flg=1;
      *type=TOKEN_RANGE;
    }
    p++;
  }
  if (*p=='\0')
    next=NULL;
  else
    next=p+1;
  if (*type==TOKEN_INT)
  {
    if (sscanf(pattern, "%i", low)!=1)
    {
      *type=-1;
      return NULL;
    }
  }
  else if (!step_flg)
  {
    *step=1;
    if (sscanf(pattern, "%i", low)!=1)
    {
      *type=-1;
      return NULL;
    }
    p=pattern;
    while (*p!=':')
      p++;
    p++;
    if (sscanf(p, "%i", high)!=1)
    {
      *type=-1;
      return NULL;
    }
  }
  else /* step is there */
  {
    if (sscanf(pattern, "%i", low)!=1)
    {
      *type=-1;
      return NULL;
    }
    p=pattern;
    while (*p!=':')
      p++;
    p++;
    if (sscanf(p, "%i", step)!=1)
    {
      *type=-1;
      return NULL;
    }
    while (*p!=':')
      p++;
    p++;
    if (sscanf(p, "%i", high)!=1)
    {
      *type=-1;
      return NULL;
    }
  }
  return next;
}

